@@ -1,5 +1,4 @@
-use ntex::codec::{AsyncRead, AsyncWrite};
-use ntex::framed::{Dispatcher as IoDispatcher, State as IoState, Timer};
+use ntex::io::{Dispatcher as IoDispatcher, IoBoxed};
 use ntex::service::{fn_service, Service};
 use ntex::time::Seconds;
 use ntex::util::Ready;
@@ -7,52 +6,42 @@ use ntex::IntoService;
 
 use crate::codec::{AmqpCodec, AmqpFrame};
 use crate::control::ControlFrame;
-use crate::error::{DispatcherError, LinkError};
+use crate::error::{AmqpDispatcherError, LinkError};
 use crate::{dispatcher::Dispatcher, Configuration, Connection, State};
 
 /// Mqtt client
-pub struct Client<Io, St = ()> {
-    io: Io,
-    state: IoState,
+pub struct Client<St = ()> {
+    io: IoBoxed,
     codec: AmqpCodec<AmqpFrame>,
     connection: Connection,
     keepalive: Seconds,
     remote_config: Configuration,
-    timer: Timer,
     _st: State<St>,
 }
 
-impl<T> Client<T, ()>
-where
-    T: AsyncRead + AsyncWrite + Unpin,
-{
+impl Client {
     /// Construct new `Dispatcher` instance with outgoing messages stream.
     pub(super) fn new(
-        io: T,
-        state: IoState,
+        io: IoBoxed,
         codec: AmqpCodec<AmqpFrame>,
         connection: Connection,
         keepalive: Seconds,
         remote_config: Configuration,
-        timer: Timer,
     ) -> Self {
         Client {
             io,
-            state,
             codec,
             connection,
             keepalive,
             remote_config,
-            timer,
             _st: State::new(()),
         }
     }
 }
 
-impl<Io, St> Client<Io, St>
+impl<St> Client<St>
 where
     St: 'static,
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
 {
     #[inline]
     /// Get client sink
@@ -62,15 +51,13 @@ where
 
     #[inline]
     /// Set connection state
-    pub fn state<T: 'static>(self, st: T) -> Client<Io, T> {
+    pub fn state<T: 'static>(self, st: T) -> Client<T> {
         Client {
             io: self.io,
-            state: self.state,
             codec: self.codec,
             connection: self.connection,
             keepalive: self.keepalive,
             remote_config: self.remote_config,
-            timer: self.timer,
             _st: State::new(st),
         }
     }
@@ -78,7 +65,7 @@ where
     /// Run client with default control messages handler.
     ///
     /// Default handler closes connection on any control message.
-    pub async fn start_default(self) -> Result<(), DispatcherError> {
+    pub async fn start_default(self) -> Result<(), AmqpDispatcherError> {
         let dispatcher = Dispatcher::new(
             self.connection,
             fn_service(|_| Ready::<_, LinkError>::Ok(())),
@@ -93,15 +80,15 @@ where
             Seconds::ZERO
         };
 
-        IoDispatcher::new(self.io, self.codec, self.state, dispatcher, self.timer)
+        IoDispatcher::new(self.io, self.codec, dispatcher)
             .keepalive_timeout(keepalive)
             .await
     }
 
-    pub async fn start<F, S>(self, service: F) -> Result<(), DispatcherError>
+    pub async fn start<F, S>(self, service: F) -> Result<(), AmqpDispatcherError>
     where
-        F: IntoService<S>,
-        S: Service<Request = ControlFrame, Response = ()>,
+        F: IntoService<S, ControlFrame>,
+        S: Service<ControlFrame, Response = ()>,
         S::Error: Into<crate::error::Error> + std::fmt::Debug + 'static,
         S: 'static,
     {
@@ -119,7 +106,7 @@ where
             Seconds::ZERO
         };
 
-        IoDispatcher::new(self.io, self.codec, self.state, dispatcher, self.timer)
+        IoDispatcher::new(self.io, self.codec, dispatcher)
             .keepalive_timeout(keepalive)
             .await
     }
