@@ -1,14 +1,13 @@
 use std::{future::Future, marker::PhantomData};
 
-use ntex::codec::{AsyncRead, AsyncWrite};
 use ntex::connect::{self, Address, Connect};
-use ntex::framed::{State, Timer};
+use ntex::io::{utils::Boxed, Filter, Io, IoBoxed};
 use ntex::service::Service;
-use ntex::time::{timeout, Millis, Seconds};
+use ntex::time::{timeout, Seconds};
 use ntex::util::{ByteString, Either, PoolId, PoolRef};
 
 #[cfg(feature = "openssl")]
-use ntex::connect::openssl::{OpensslConnector, SslConnector};
+use ntex::connect::openssl::{self, OpensslConnector, SslConnector};
 
 #[cfg(feature = "rustls")]
 use ntex::connect::rustls::{ClientConfig, RustlsConnector};
@@ -20,27 +19,25 @@ use crate::{error::ProtocolIdError, Configuration, Connection};
 use super::{connection::Client, error::ConnectError, SaslAuth};
 
 /// Amqp client connector
-pub struct Connector<A, T> {
+pub struct Connector<A, T = ()> {
     connector: T,
     config: Configuration,
     handshake_timeout: Seconds,
     disconnect_timeout: Seconds,
     pool: PoolRef,
-    timer: Timer,
     _t: PhantomData<A>,
 }
 
 impl<A> Connector<A, ()> {
     #[allow(clippy::new_ret_no_self)]
     /// Create new amqp connector
-    pub fn new() -> Connector<A, connect::Connector<A>> {
+    pub fn new() -> Connector<A, Boxed<connect::Connector<A>, Connect<A>>> {
         Connector {
-            connector: connect::Connector::default(),
+            connector: Boxed::new(connect::Connector::default()),
             handshake_timeout: Seconds::ZERO,
             disconnect_timeout: Seconds(3),
             config: Configuration::default(),
             pool: PoolId::P6.pool_ref(),
-            timer: Timer::new(Millis::ONE_SEC),
             _t: PhantomData,
         }
     }
@@ -49,8 +46,6 @@ impl<A> Connector<A, ()> {
 impl<A, T> Connector<A, T>
 where
     A: Address,
-    T: Service<Request = Connect<A>, Error = connect::ConnectError>,
-    T::Response: AsyncRead + AsyncWrite + Unpin + 'static,
 {
     /// The channel-max value is the highest channel number that
     /// may be used on the Connection. This value plus one is the maximum
@@ -122,26 +117,60 @@ where
         self
     }
 
-    #[doc(hidden)]
-    #[deprecated(since = "0.5.6", note = "Use memory pool config")]
-    #[inline]
-    /// Set read/write buffer params
-    ///
-    /// By default read buffer is 8kb, write buffer is 8kb
-    pub fn buffer_params(
+    /// Use custom connector
+    pub fn connector<U, F>(self, connector: U) -> Connector<A, Boxed<U, Connect<A>>>
+    where
+        F: Filter,
+        U: Service<Connect<A>, Response = Io<F>, Error = connect::ConnectError>,
+    {
+        Connector {
+            connector: Boxed::new(connector),
+            config: self.config,
+            handshake_timeout: self.handshake_timeout,
+            disconnect_timeout: self.disconnect_timeout,
+            pool: self.pool,
+            _t: PhantomData,
+        }
+    }
+
+    #[cfg(feature = "openssl")]
+    /// Use openssl connector
+    pub fn openssl(
         self,
-        _max_read_buf_size: u16,
-        _max_write_buf_size: u16,
-        _min_buf_size: u16,
-    ) -> Self {
-        self
+        connector: SslConnector,
+    ) -> Connector<A, Boxed<openssl::Connector<A>, Connect<A>>> {
+        Connector {
+            config: self.config,
+            connector: Boxed::new(openssl::IoConnector(connector)),
+            handshake_timeout: self.handshake_timeout,
+            disconnect_timeout: self.disconnect_timeout,
+            pool: self.pool,
+            _t: PhantomData,
+        }
+    }
+
+    #[cfg(feature = "rustls")]
+    /// Use rustls connector
+    pub fn rustls(
+        self,
+        config: ClientConfig,
+    ) -> Connector<A, Boxed<rustls::Connector<A>, Connect<A>>> {
+        use std::sync::Arc;
+
+        Connector {
+            config: self.config,
+            connector: Boxed::new(rustls::Connector::new(Arc::new(config))),
+            handshake_timeout: self.handshake_timeout,
+            disconnect_timeout: self.disconnect_timeout,
+            pool: self.pool,
+            _t: PhantomData,
+        }
     }
 
     /// Use custom connector
-    pub fn connector<U>(self, connector: U) -> Connector<A, U>
+    pub fn boxed_connector<U>(self, connector: U) -> Connector<A, U>
     where
-        U: Service<Request = Connect<A>, Error = connect::ConnectError>,
-        U::Response: AsyncRead + AsyncWrite + Unpin + 'static,
+        U: Service<Connect<A>, Response = IoBoxed, Error = connect::ConnectError>,
     {
         Connector {
             connector,
@@ -149,46 +178,18 @@ where
             handshake_timeout: self.handshake_timeout,
             disconnect_timeout: self.disconnect_timeout,
             pool: self.pool,
-            timer: self.timer,
-            _t: PhantomData,
-        }
-    }
-
-    #[cfg(feature = "openssl")]
-    /// Use openssl connector
-    pub fn openssl(self, connector: SslConnector) -> Connector<A, OpensslConnector<A>> {
-        Connector {
-            config: self.config,
-            connector: OpensslConnector::new(connector),
-            handshake_timeout: self.handshake_timeout,
-            disconnect_timeout: self.disconnect_timeout,
-            pool: self.pool,
-            timer: self.timer,
-            _t: PhantomData,
-        }
-    }
-
-    #[cfg(feature = "rustls")]
-    /// Use rustls connector
-    pub fn rustls(self, config: ClientConfig) -> Connector<A, RustlsConnector<A>> {
-        use std::sync::Arc;
-
-        Connector {
-            config: self.config,
-            connector: RustlsConnector::new(Arc::new(config)),
-            handshake_timeout: self.handshake_timeout,
-            disconnect_timeout: self.disconnect_timeout,
-            pool: self.pool,
-            timer: self.timer,
             _t: PhantomData,
         }
     }
+}
 
+impl<A, T> Connector<A, T>
+where
+    A: Address,
+    T: Service<Connect<A>, Response = IoBoxed, Error = connect::ConnectError>,
+{
     /// Connect to amqp server
-    pub fn connect(
-        &self,
-        address: A,
-    ) -> impl Future<Output = Result<Client<T::Response>, ConnectError>> {
+    pub fn connect(&self, address: A) -> impl Future<Output = Result<Client, ConnectError>> {
         if self.handshake_timeout.non_zero() {
             let fut = timeout(self.handshake_timeout, self._connect(address));
             Either::Left(async move {
@@ -203,33 +204,29 @@ where
     }
 
     /// Negotiate amqp protocol over opened socket
-    pub fn negotiate<Io>(&self, io: Io) -> impl Future<Output = Result<Client<Io>, ConnectError>>
-    where
-        Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    {
+    pub fn negotiate(&self, io: IoBoxed) -> impl Future<Output = Result<Client, ConnectError>> {
         trace!("Negotiation client protocol id: Amqp");
 
-        let state = State::with_memory_pool(self.pool);
-        state.set_disconnect_timeout(self.disconnect_timeout);
+        io.set_memory_pool(self.pool);
+        io.set_disconnect_timeout(self.disconnect_timeout.into());
 
-        _connect_plain(io, state, self.config.clone(), self.timer.clone())
+        _connect_plain(io, self.config.clone())
     }
 
-    fn _connect(
-        &self,
-        address: A,
-    ) -> impl Future<Output = Result<Client<T::Response>, ConnectError>> {
+    fn _connect(&self, address: A) -> impl Future<Output = Result<Client, ConnectError>> {
         let fut = self.connector.call(Connect::new(address));
         let config = self.config.clone();
-        let timer = self.timer.clone();
-        let state = State::with_memory_pool(self.pool);
-        state.set_disconnect_timeout(self.disconnect_timeout);
+        let pool = self.pool;
+        let disconnect = self.disconnect_timeout;
 
         async move {
             trace!("Negotiation client protocol id: Amqp");
 
-            let io = fut.await?;
-            _connect_plain(io, state, config, timer).await
+            let state = fut.await?;
+            state.set_memory_pool(pool);
+            state.set_disconnect_timeout(disconnect.into());
+
+            _connect_plain(state, config).await
         }
     }
 
@@ -238,7 +235,7 @@ where
         &self,
         addr: A,
         auth: SaslAuth,
-    ) -> impl Future<Output = Result<Client<T::Response>, ConnectError>> {
+    ) -> impl Future<Output = Result<Client, ConnectError>> {
         if self.handshake_timeout.non_zero() {
             let fut = timeout(self.handshake_timeout, self._connect_sasl(addr, auth));
             Either::Left(async move {
@@ -253,65 +250,54 @@ where
     }
 
     /// Negotiate amqp sasl protocol over opened socket
-    pub fn negotiate_sasl<Io>(
+    pub fn negotiate_sasl(
         &self,
-        io: Io,
+        io: IoBoxed,
         auth: SaslAuth,
-    ) -> impl Future<Output = Result<Client<Io>, ConnectError>>
-    where
-        Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    {
+    ) -> impl Future<Output = Result<Client, ConnectError>> {
         trace!("Negotiation client protocol id: Amqp");
 
         let config = self.config.clone();
-        let timer = self.timer.clone();
-        let state = State::with_memory_pool(self.pool);
-        state.set_disconnect_timeout(self.disconnect_timeout);
+        io.set_memory_pool(self.pool);
+        io.set_disconnect_timeout(self.disconnect_timeout.into());
 
-        _connect_sasl(io, state, auth, config, timer)
+        _connect_sasl(io, auth, config)
     }
 
     fn _connect_sasl(
         &self,
         addr: A,
         auth: SaslAuth,
-    ) -> impl Future<Output = Result<Client<T::Response>, ConnectError>> {
+    ) -> impl Future<Output = Result<Client, ConnectError>> {
         let fut = self.connector.call(Connect::new(addr));
         let config = self.config.clone();
-        let timer = self.timer.clone();
-        let state = State::with_memory_pool(self.pool);
-        state.set_disconnect_timeout(self.disconnect_timeout);
+        let pool = self.pool;
+        let disconnect = self.disconnect_timeout;
 
-        async move { _connect_sasl(fut.await?, state, auth, config, timer).await }
+        async move {
+            let state = fut.await?;
+            state.set_memory_pool(pool);
+            state.set_disconnect_timeout(disconnect.into());
+
+            _connect_sasl(state, auth, config).await
+        }
     }
 }
 
-async fn _connect_sasl<T>(
-    mut io: T,
-    state: State,
+async fn _connect_sasl(
+    io: IoBoxed,
     auth: SaslAuth,
     config: Configuration,
-    timer: Timer,
-) -> Result<Client<T>, ConnectError>
-where
-    T: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+) -> Result<Client, ConnectError> {
     trace!("Negotiation client protocol id: AmqpSasl");
 
-    state
-        .send(&mut io, &ProtocolIdCodec, ProtocolId::AmqpSasl)
-        .await?;
+    io.send(ProtocolId::AmqpSasl, &ProtocolIdCodec).await?;
+
+    let proto = io.recv(&ProtocolIdCodec).await?.ok_or_else(|| {
+        log::trace!("Amqp server is disconnected during handshake");
+        ConnectError::Disconnected
+    })?;
 
-    let proto = state
-        .next(&mut io, &ProtocolIdCodec)
-        .await
-        .map_err(ConnectError::from)
-        .and_then(|res| {
-            res.ok_or_else(|| {
-                log::trace!("Amqp server is disconnected during handshake");
-                ConnectError::Disconnected
-            })
-        })?;
     if proto != ProtocolId::AmqpSasl {
         return Err(ConnectError::from(ProtocolIdError::Unexpected {
             exp: ProtocolId::AmqpSasl,
@@ -322,11 +308,7 @@ where
     let codec = AmqpCodec::<SaslFrame>::new();
 
     // processing sasl-mechanisms
-    let _ = state
-        .next(&mut io, &codec)
-        .await
-        .map_err(ConnectError::from)
-        .and_then(|res| res.ok_or(ConnectError::Disconnected))?;
+    let _ = io.recv(&codec).await?.ok_or(ConnectError::Disconnected)?;
 
     let initial_response =
         SaslInit::prepare_response(&auth.authz_id, &auth.authn_id, &auth.password);
@@ -337,14 +319,10 @@ where
         initial_response: Some(initial_response),
     };
 
-    state.send(&mut io, &codec, sasl_init.into()).await?;
+    io.send(sasl_init.into(), &codec).await?;
 
     // processing sasl-outcome
-    let sasl_frame = state
-        .next(&mut io, &codec)
-        .await
-        .map_err(ConnectError::from)
-        .and_then(|res| res.ok_or(ConnectError::Disconnected))?;
+    let sasl_frame = io.recv(&codec).await?.ok_or(ConnectError::Disconnected)?;
 
     if let SaslFrame {
         body: SaslFrameBody::SaslOutcome(outcome),
@@ -357,33 +335,21 @@ where
         return Err(ConnectError::Disconnected);
     }
 
-    _connect_plain(io, state, config, timer).await
+    _connect_plain(io, config).await
 }
 
-async fn _connect_plain<T>(
-    mut io: T,
-    state: State,
-    config: Configuration,
-    timer: Timer,
-) -> Result<Client<T>, ConnectError>
-where
-    T: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+async fn _connect_plain(io: IoBoxed, config: Configuration) -> Result<Client, ConnectError> {
     trace!("Negotiation client protocol id: Amqp");
 
-    state
-        .send(&mut io, &ProtocolIdCodec, ProtocolId::Amqp)
-        .await?;
+    io.send(ProtocolId::Amqp, &ProtocolIdCodec).await?;
 
-    let proto = state
-        .next(&mut io, &ProtocolIdCodec)
+    let proto = io
+        .recv(&ProtocolIdCodec)
         .await
-        .map_err(ConnectError::from)
-        .and_then(|res| {
-            res.ok_or_else(|| {
-                log::trace!("Amqp server is disconnected during handshake");
-                ConnectError::Disconnected
-            })
+        .map_err(ConnectError::from)?
+        .ok_or_else(|| {
+            log::trace!("Amqp server is disconnected during handshake");
+            ConnectError::Disconnected
         })?;
 
     if proto != ProtocolId::Amqp {
@@ -397,34 +363,19 @@ where
     let codec = AmqpCodec::<AmqpFrame>::new().max_size(config.max_frame_size as usize);
 
     trace!("Open client amqp connection: {:?}", open);
-    state
-        .send(&mut io, &codec, AmqpFrame::new(0, Frame::Open(open)))
+    io.send(AmqpFrame::new(0, Frame::Open(open)), &codec)
         .await?;
 
-    let frame = state
-        .next(&mut io, &codec)
-        .await
-        .map_err(ConnectError::from)
-        .and_then(|res| {
-            res.ok_or_else(|| {
-                log::trace!("Amqp server is disconnected during handshake");
-                ConnectError::Disconnected
-            })
-        })?;
+    let frame = io.recv(&codec).await?.ok_or_else(|| {
+        log::trace!("Amqp server is disconnected during handshake");
+        ConnectError::Disconnected
+    })?;
 
     if let Frame::Open(open) = frame.performative() {
         trace!("Open confirmed: {:?}", open);
         let remote_config = open.into();
-        let connection = Connection::new(state.clone(), &config, &remote_config);
-        let client = Client::new(
-            io,
-            state,
-            codec,
-            connection,
-            config.timeout_secs(),
-            remote_config,
-            timer,
-        );
+        let connection = Connection::new(io.get_ref(), &config, &remote_config);
+        let client = Client::new(io, codec, connection, config.timeout_secs(), remote_config);
         Ok(client)
     } else {
         Err(ConnectError::ExpectOpenFrame(Box::new(frame)))
