@@ -1,10 +1,8 @@
 use std::{fmt, future::Future, marker, pin::Pin, rc::Rc, task::Context, task::Poll};
 
-use ntex::codec::{AsyncRead, AsyncWrite};
-use ntex::framed::{Dispatcher as FramedDispatcher, State as IoState, Timer};
+use ntex::io::{Dispatcher as FramedDispatcher, Filter, Io, IoBoxed};
 use ntex::service::{IntoServiceFactory, Service, ServiceFactory};
-use ntex::time::{timeout, Millis, Seconds};
-use ntex::util::{Pool, PoolId, PoolRef};
+use ntex::time::{timeout, Seconds};
 
 use crate::codec::{protocol::ProtocolId, AmqpCodec, AmqpFrame, ProtocolIdCodec, ProtocolIdError};
 use crate::{default::DefaultControlService, Configuration, Connection, ControlFrame, State};
@@ -13,16 +11,21 @@ use crate::{dispatcher::Dispatcher, types::Message};
 use super::handshake::{Handshake, HandshakeAck};
 use super::{Error, HandshakeError, ServerError};
 
-/// Server dispatcher factory
-pub struct Server<Io, St, H, Ctl> {
+/// Amqp server factory
+pub struct Server<St, H, Ctl, Pb> {
+    handshake: H,
+    inner: Rc<ServerInner<St, Ctl, Pb>>,
+}
+
+/// Amqp server builder
+pub struct ServerBuilder<St, H, Ctl> {
     handshake: H,
     control: Ctl,
     config: Rc<Configuration>,
     max_size: usize,
     handshake_timeout: Seconds,
     disconnect_timeout: Seconds,
-    pool: PoolRef,
-    _t: marker::PhantomData<(Io, St)>,
+    _t: marker::PhantomData<St>,
 }
 
 pub(super) struct ServerInner<St, Ctl, Pb> {
@@ -32,37 +35,32 @@ pub(super) struct ServerInner<St, Ctl, Pb> {
     max_size: usize,
     handshake_timeout: Seconds,
     disconnect_timeout: Seconds,
-    time: Timer,
     _t: marker::PhantomData<St>,
 }
 
-impl<Io, St, H> Server<Io, St, H, DefaultControlService<St, H::Error>>
+impl<St> Server<St, (), (), ()>
 where
     St: 'static,
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    H: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
-        + 'static,
-    H::Error: fmt::Debug,
 {
-    /// Create server factory and provide handshake service
-    pub fn new<F>(handshake: F) -> Self
+    /// Start server buldeing process with provided handshake service
+    pub fn build<F, H>(handshake: F) -> ServerBuilder<St, H, DefaultControlService<St, H::Error>>
     where
-        F: IntoServiceFactory<H>,
+        F: IntoServiceFactory<H, Handshake>,
+        H: ServiceFactory<Handshake, Response = HandshakeAck<St>>,
     {
-        Self {
+        ServerBuilder {
             handshake: handshake.into_factory(),
             handshake_timeout: Seconds(5),
             disconnect_timeout: Seconds(3),
             control: DefaultControlService::default(),
             max_size: 0,
             config: Rc::new(Configuration::default()),
-            pool: PoolId::P6.pool_ref(),
             _t: marker::PhantomData,
         }
     }
 }
 
-impl<Io, St, H, Ctl> Server<Io, St, H, Ctl> {
+impl<St, H, Ctl> ServerBuilder<St, H, Ctl> {
     /// Provide connection configuration
     pub fn config(mut self, config: Configuration) -> Self {
         self.config = Rc::new(config);
@@ -98,85 +96,48 @@ impl<Io, St, H, Ctl> Server<Io, St, H, Ctl> {
         self.disconnect_timeout = val;
         self
     }
-
-    /// Set memory pool.
-    ///
-    /// Use specified memory pool for memory allocations. By default P6
-    /// memory pool is used.
-    pub fn memory_pool(mut self, id: PoolId) -> Self {
-        self.pool = id.pool_ref();
-        self
-    }
-
-    #[doc(hidden)]
-    #[deprecated(since = "0.5.6", note = "Use memory pool config")]
-    #[inline]
-    /// Set read/write buffer params
-    ///
-    /// By default read buffer is 8kb, write buffer is 8kb
-    pub fn buffer_params(
-        self,
-        _max_read_buf_size: u16,
-        _max_write_buf_size: u16,
-        _min_buf_size: u16,
-    ) -> Self {
-        self
-    }
 }
 
-impl<Io, St, H, Ctl> Server<Io, St, H, Ctl>
+impl<St, H, Ctl> ServerBuilder<St, H, Ctl>
 where
     St: 'static,
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    H: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
-        + 'static,
+    H: ServiceFactory<Handshake, Response = HandshakeAck<St>> + 'static,
     H::Error: fmt::Debug,
-    Ctl: ServiceFactory<Config = State<St>, Request = ControlFrame, Response = ()> + 'static,
+    Ctl: ServiceFactory<ControlFrame, State<St>, Response = ()> + 'static,
     Ctl::Error: fmt::Debug,
     Ctl::InitError: fmt::Debug,
     Error: From<Ctl::Error>,
 {
     /// Service to call with control frames
-    pub fn control<F, S>(self, service: F) -> Server<Io, St, H, S>
+    pub fn control<F, S>(self, service: F) -> ServerBuilder<St, H, S>
     where
-        F: IntoServiceFactory<S>,
-        S: ServiceFactory<Config = State<St>, Request = ControlFrame, Response = ()> + 'static,
+        F: IntoServiceFactory<S, ControlFrame, State<St>>,
+        S: ServiceFactory<ControlFrame, State<St>, Response = ()> + 'static,
         S::Error: fmt::Debug,
         S::InitError: fmt::Debug,
         Error: From<S::Error>,
     {
-        Server {
+        ServerBuilder {
             config: self.config,
             handshake: self.handshake,
             handshake_timeout: self.handshake_timeout,
             disconnect_timeout: self.disconnect_timeout,
             control: service.into_factory(),
-            pool: self.pool,
             max_size: self.max_size,
             _t: marker::PhantomData,
         }
     }
 
     /// Set service to execute for incoming links and create service factory
-    pub fn finish<F, Pb>(
-        self,
-        service: F,
-    ) -> impl ServiceFactory<
-        Config = (),
-        Request = Io,
-        Response = (),
-        Error = ServerError<H::Error>,
-        InitError = H::InitError,
-    >
+    pub fn finish<S, Pb>(self, service: S) -> Server<St, H, Ctl, Pb>
     where
-        F: IntoServiceFactory<Pb>,
-        Pb: ServiceFactory<Config = State<St>, Request = Message, Response = ()> + 'static,
+        S: IntoServiceFactory<Pb, Message, State<St>>,
+        Pb: ServiceFactory<Message, State<St>, Response = ()> + 'static,
         Pb::Error: fmt::Debug,
         Pb::InitError: fmt::Debug,
         Error: From<Pb::Error> + From<Ctl::Error>,
     {
-        ServerImpl {
-            pool: self.pool,
+        Server {
             handshake: self.handshake,
             inner: Rc::new(ServerInner {
                 handshake_timeout: self.handshake_timeout,
@@ -185,123 +146,115 @@ where
                 control: self.control,
                 disconnect_timeout: self.disconnect_timeout,
                 max_size: self.max_size,
-                time: Timer::new(Millis::ONE_SEC),
                 _t: marker::PhantomData,
             }),
-            _t: marker::PhantomData,
         }
     }
 }
 
-struct ServerImpl<Io, St, H, Ctl, Pb> {
-    handshake: H,
-    pool: PoolRef,
-    inner: Rc<ServerInner<St, Ctl, Pb>>,
-    _t: marker::PhantomData<Io>,
-}
-
-impl<Io, St, H, Ctl, Pb> ServiceFactory for ServerImpl<Io, St, H, Ctl, Pb>
+impl<F, St, H, Ctl, Pb> ServiceFactory<Io<F>> for Server<St, H, Ctl, Pb>
 where
+    F: Filter,
     St: 'static,
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    H: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
-        + 'static,
+    H: ServiceFactory<Handshake, Response = HandshakeAck<St>> + 'static,
     H::Error: fmt::Debug,
-    Ctl: ServiceFactory<Config = State<St>, Request = ControlFrame, Response = ()> + 'static,
+    Ctl: ServiceFactory<ControlFrame, State<St>, Response = ()> + 'static,
     Ctl::Error: fmt::Debug,
     Ctl::InitError: fmt::Debug,
-    Pb: ServiceFactory<Config = State<St>, Request = Message, Response = ()> + 'static,
+    Pb: ServiceFactory<Message, State<St>, Response = ()> + 'static,
     Pb::Error: fmt::Debug,
     Pb::InitError: fmt::Debug,
     Error: From<Pb::Error> + From<Ctl::Error>,
 {
-    type Config = ();
-    type Request = Io;
     type Response = ();
     type Error = ServerError<H::Error>;
-    type Service = ServerImplService<Io, St, H::Service, Ctl, Pb>;
+    type Service = ServerHandler<St, H::Service, Ctl, Pb>;
     type InitError = H::InitError;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
-        let pool = self.pool.pool();
         let inner = self.inner.clone();
         let fut = self.handshake.new_service(());
 
         Box::pin(async move {
-            fut.await.map(move |handshake| ServerImplService {
+            fut.await.map(move |handshake| ServerHandler {
                 inner,
-                pool,
                 handshake: Rc::new(handshake),
-                _t: marker::PhantomData,
             })
         })
     }
 }
 
-struct ServerImplService<Io, St, H, Ctl, Pb> {
-    handshake: Rc<H>,
-    pool: Pool,
-    inner: Rc<ServerInner<St, Ctl, Pb>>,
-    _t: marker::PhantomData<Io>,
-}
-
-impl<Io, St, H, Ctl, Pb> Service for ServerImplService<Io, St, H, Ctl, Pb>
+impl<St, H, Ctl, Pb> ServiceFactory<IoBoxed> for Server<St, H, Ctl, Pb>
 where
     St: 'static,
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    H: Service<Request = Handshake<Io>, Response = HandshakeAck<Io, St>> + 'static,
+    H: ServiceFactory<Handshake, Response = HandshakeAck<St>> + 'static,
     H::Error: fmt::Debug,
-    Ctl: ServiceFactory<Config = State<St>, Request = ControlFrame, Response = ()> + 'static,
+    Ctl: ServiceFactory<ControlFrame, State<St>, Response = ()> + 'static,
     Ctl::Error: fmt::Debug,
     Ctl::InitError: fmt::Debug,
-    Pb: ServiceFactory<Config = State<St>, Request = Message, Response = ()> + 'static,
+    Pb: ServiceFactory<Message, State<St>, Response = ()> + 'static,
     Pb::Error: fmt::Debug,
     Pb::InitError: fmt::Debug,
     Error: From<Pb::Error> + From<Ctl::Error>,
 {
-    type Request = Io;
     type Response = ();
     type Error = ServerError<H::Error>;
-    type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>>>>;
+    type Service = ServerHandler<St, H::Service, Ctl, Pb>;
+    type InitError = H::InitError;
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
-    #[inline]
-    fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
-        let ready1 = self.pool.poll_ready(cx).is_ready();
-        let ready2 = self
-            .handshake
-            .as_ref()
-            .poll_ready(cx)
-            .map(|res| res.map_err(ServerError::Service))?
-            .is_ready();
+    fn new_service(&self, _: ()) -> Self::Future {
+        let inner = self.inner.clone();
+        let fut = self.handshake.new_service(());
 
-        if ready1 && ready2 {
-            Poll::Ready(Ok(()))
-        } else {
-            Poll::Pending
-        }
+        Box::pin(async move {
+            fut.await.map(move |handshake| ServerHandler {
+                inner,
+                handshake: Rc::new(handshake),
+            })
+        })
     }
+}
 
-    #[inline]
-    fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
-        self.handshake.as_ref().poll_shutdown(cx, is_error)
-    }
+/// Amqp connections handler
+pub struct ServerHandler<St, H, Ctl, Pb> {
+    handshake: Rc<H>,
+    inner: Rc<ServerInner<St, Ctl, Pb>>,
+}
+
+impl<St, H, Ctl, Pb> ServerHandler<St, H, Ctl, Pb>
+where
+    St: 'static,
+    H: Service<Handshake, Response = HandshakeAck<St>> + 'static,
+    H::Error: fmt::Debug,
+    Ctl: ServiceFactory<ControlFrame, State<St>, Response = ()> + 'static,
+    Ctl::Error: fmt::Debug,
+    Ctl::InitError: fmt::Debug,
+    Pb: ServiceFactory<Message, State<St>, Response = ()> + 'static,
+    Pb::Error: fmt::Debug,
+    Pb::InitError: fmt::Debug,
+    Error: From<Pb::Error> + From<Ctl::Error>,
+{
+    fn create(
+        &self,
+        req: IoBoxed,
+    ) -> Pin<Box<dyn Future<Output = Result<(), ServerError<H::Error>>>>> {
+        req.set_disconnect_timeout(self.inner.disconnect_timeout.into());
 
-    fn call(&self, req: Self::Request) -> Self::Future {
         let keepalive = self.inner.config.idle_time_out / 1000;
         let handshake_timeout = self.inner.handshake_timeout;
         let disconnect_timeout = self.inner.disconnect_timeout;
-        let inner = self.inner.clone();
         let fut = handshake(
             req,
             self.inner.max_size,
             self.handshake.clone(),
-            self.pool.pool_ref(),
             self.inner.clone(),
         );
+        let inner = self.inner.clone();
 
         Box::pin(async move {
-            let (io, state, codec, sink, st, idle_timeout) = if handshake_timeout.is_zero() {
+            let (state, codec, sink, st, idle_timeout) = if handshake_timeout.is_zero() {
                 fut.await?
             } else {
                 timeout(handshake_timeout, fut)
@@ -324,7 +277,7 @@ where
             let dispatcher = Dispatcher::new(sink, pb_srv, ctl_srv, idle_timeout.into())
                 .map(|_| Option::<AmqpFrame>::None);
 
-            FramedDispatcher::new(io, codec, state, dispatcher, inner.time.clone())
+            FramedDispatcher::new(state, codec, dispatcher)
                 .keepalive_timeout(Seconds::checked_new(keepalive as usize))
                 .disconnect_timeout(disconnect_timeout)
                 .await
@@ -333,16 +286,85 @@ where
     }
 }
 
-async fn handshake<Io, St, H, Ctl, Pb>(
-    mut io: Io,
+impl<F, St, H, Ctl, Pb> Service<Io<F>> for ServerHandler<St, H, Ctl, Pb>
+where
+    F: Filter,
+    St: 'static,
+    H: Service<Handshake, Response = HandshakeAck<St>> + 'static,
+    H::Error: fmt::Debug,
+    Ctl: ServiceFactory<ControlFrame, State<St>, Response = ()> + 'static,
+    Ctl::Error: fmt::Debug,
+    Ctl::InitError: fmt::Debug,
+    Pb: ServiceFactory<Message, State<St>, Response = ()> + 'static,
+    Pb::Error: fmt::Debug,
+    Pb::InitError: fmt::Debug,
+    Error: From<Pb::Error> + From<Ctl::Error>,
+{
+    type Response = ();
+    type Error = ServerError<H::Error>;
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>>>>;
+
+    #[inline]
+    fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
+        self.handshake
+            .as_ref()
+            .poll_ready(cx)
+            .map(|res| res.map_err(ServerError::Service))
+    }
+
+    #[inline]
+    fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
+        self.handshake.as_ref().poll_shutdown(cx, is_error)
+    }
+
+    fn call(&self, req: Io<F>) -> Self::Future {
+        self.create(IoBoxed::from(req))
+    }
+}
+
+impl<St, H, Ctl, Pb> Service<IoBoxed> for ServerHandler<St, H, Ctl, Pb>
+where
+    St: 'static,
+    H: Service<Handshake, Response = HandshakeAck<St>> + 'static,
+    H::Error: fmt::Debug,
+    Ctl: ServiceFactory<ControlFrame, State<St>, Response = ()> + 'static,
+    Ctl::Error: fmt::Debug,
+    Ctl::InitError: fmt::Debug,
+    Pb: ServiceFactory<Message, State<St>, Response = ()> + 'static,
+    Pb::Error: fmt::Debug,
+    Pb::InitError: fmt::Debug,
+    Error: From<Pb::Error> + From<Ctl::Error>,
+{
+    type Response = ();
+    type Error = ServerError<H::Error>;
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>>>>;
+
+    #[inline]
+    fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
+        self.handshake
+            .as_ref()
+            .poll_ready(cx)
+            .map(|res| res.map_err(ServerError::Service))
+    }
+
+    #[inline]
+    fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
+        self.handshake.as_ref().poll_shutdown(cx, is_error)
+    }
+
+    fn call(&self, req: IoBoxed) -> Self::Future {
+        self.create(req)
+    }
+}
+
+async fn handshake<St, H, Ctl, Pb>(
+    state: IoBoxed,
     max_size: usize,
     handshake: Rc<H>,
-    pool: PoolRef,
     inner: Rc<ServerInner<St, Ctl, Pb>>,
 ) -> Result<
     (
-        Io,
-        IoState,
+        IoBoxed,
         AmqpCodec<AmqpFrame>,
         Connection,
         State<St>,
@@ -352,16 +374,12 @@ async fn handshake<Io, St, H, Ctl, Pb>(
 >
 where
     St: 'static,
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    H: Service<Request = Handshake<Io>, Response = HandshakeAck<Io, St>>,
-    Ctl: ServiceFactory<Config = State<St>, Request = ControlFrame, Response = ()> + 'static,
-    Pb: ServiceFactory<Config = State<St>, Request = Message, Response = ()> + 'static,
+    H: Service<Handshake, Response = HandshakeAck<St>>,
+    Ctl: ServiceFactory<ControlFrame, State<St>, Response = ()> + 'static,
+    Pb: ServiceFactory<Message, State<St>, Response = ()> + 'static,
 {
-    let state = IoState::with_memory_pool(pool);
-    state.set_disconnect_timeout(inner.disconnect_timeout);
-
     let protocol = state
-        .next(&mut io, &ProtocolIdCodec)
+        .recv(&ProtocolIdCodec)
         .await
         .map_err(HandshakeError::from)?
         .ok_or_else(|| {
@@ -369,37 +387,37 @@ where
             HandshakeError::Disconnected
         })?;
 
-    let (io, sink, state, codec, st, idle_timeout) = match protocol {
+    let (sink, state, codec, st, idle_timeout) = match protocol {
         // start amqp processing
         ProtocolId::Amqp | ProtocolId::AmqpSasl => {
             state
-                .send(&mut io, &ProtocolIdCodec, protocol)
+                .send(protocol, &ProtocolIdCodec)
                 .await
                 .map_err(HandshakeError::from)?;
 
             let ack = handshake
                 .call(if protocol == ProtocolId::Amqp {
-                    Handshake::new_plain(io, state, inner.config.clone())
+                    Handshake::new_plain(state, inner.config.clone())
                 } else {
-                    Handshake::new_sasl(io, state, inner.config.clone())
+                    Handshake::new_sasl(state, inner.config.clone())
                 })
                 .await
                 .map_err(ServerError::Service)?;
 
-            let (st, mut io, sink, state, idle_timeout) = ack.into_inner();
+            let (st, sink, idle_timeout, state) = ack.into_inner();
 
             let codec = AmqpCodec::new().max_size(max_size);
 
             // confirm Open
             let local = inner.config.to_open();
             state
-                .send(&mut io, &codec, AmqpFrame::new(0, local.into()))
+                .send(AmqpFrame::new(0, local.into()), &codec)
                 .await
                 .map_err(HandshakeError::from)?;
 
             let st = State::new(st);
 
-            (io, sink, state, codec, st, idle_timeout)
+            (sink, state, codec, st, idle_timeout)
         }
         ProtocolId::AmqpTls => {
             return Err(HandshakeError::from(ProtocolIdError::Unexpected {
@@ -410,5 +428,5 @@ where
         }
     };
 
-    Ok((io, state, codec, sink, st, idle_timeout))
+    Ok((state, codec, sink, st, idle_timeout))
 }
